@@ -1,4 +1,3 @@
-
 /**************************************************************************
 
 Copyright 1998-1999 Precision Insight, Inc., Cedar Park, Texas.
@@ -97,6 +96,7 @@ Bool I830XAAInit(ScreenPtr pScreen);
 typedef struct _I830OutputRec I830OutputRec, *I830OutputPtr;
 
 #include "common.h"
+#include "i830_ring.h"
 #include "i830_sdvo.h"
 #include "i2c_vid.h"
 
@@ -118,14 +118,21 @@ typedef struct _I830OutputRec I830OutputRec, *I830OutputPtr;
 #define I830_KERNEL_TEX (1 << 1) /* Allocate texture memory pool */
 #endif
 
+#ifdef XvMCExtension
+#ifdef ENABLE_XVMC
+#define INTEL_XVMC 1
+#endif
+#endif
+
 typedef struct _I830Rec *I830Ptr;
 
 typedef void (*I830WriteIndexedByteFunc)(I830Ptr pI830, IOADDRESS addr,
-                                         CARD8 index, CARD8 value);
-typedef CARD8(*I830ReadIndexedByteFunc)(I830Ptr pI830, IOADDRESS addr,
-                                        CARD8 index);
-typedef void (*I830WriteByteFunc)(I830Ptr pI830, IOADDRESS addr, CARD8 value);
-typedef CARD8(*I830ReadByteFunc)(I830Ptr pI830, IOADDRESS addr);
+                                         uint8_t index, uint8_t value);
+typedef uint8_t(*I830ReadIndexedByteFunc)(I830Ptr pI830, IOADDRESS addr,
+					  uint8_t index);
+typedef void (*I830WriteByteFunc)(I830Ptr pI830, IOADDRESS addr,
+				  uint8_t value);
+typedef uint8_t(*I830ReadByteFunc)(I830Ptr pI830, IOADDRESS addr);
 
 enum tile_format {
     TILE_NONE,
@@ -259,7 +266,7 @@ typedef struct _I830CrtcPrivateRec {
     int			    dpms_mode;
     
     /* Lookup table values to be set when the CRTC is enabled */
-    CARD8 lut_r[256], lut_g[256], lut_b[256];
+    uint8_t lut_r[256], lut_g[256], lut_b[256];
 
     i830_memory *rotate_mem;
     /* Card virtual address of the cursor */
@@ -394,10 +401,23 @@ typedef struct _I830Rec {
    /* Regions allocated either from the above pools, or from agpgart. */
    I830RingBuffer *LpRing;
 
+   /** Number of bytes being emitted in the current BEGIN_LP_RING */
+   unsigned int ring_emitting;
+   /** Number of bytes that have been emitted in the current BEGIN_LP_RING */
+   unsigned int ring_used;
+   /** Offset in the ring for the next DWORD emit */
+   uint32_t ring_next;
+
 #ifdef I830_XV
    /* For Xvideo */
    i830_memory *overlay_regs;
 #endif
+#ifdef INTEL_XVMC
+   /* For XvMC */
+   Bool XvMCEnabled;
+   Bool IsXvMCSurface;
+#endif
+
    XF86ModReqInfo shadowReq; /* to test for later libshadow */
    Rotation rotation;
    void (*PointerMoved)(int, int, int);
@@ -450,7 +470,7 @@ typedef struct _I830Rec {
    pciVideoPtr PciInfo;
    PCITAG PciTag;
 #endif
-   CARD8 variant;
+   uint8_t variant;
 
    unsigned int BR[20];
 
@@ -504,9 +524,10 @@ typedef struct _I830Rec {
    float scale_units[2][2];
   /** Transform pointers for src/mask, or NULL if identity */
    PictTransform *transform[2];
+   float coord_adjust;
    /* i915 EXA render state */
-   CARD32 mapstate[6];
-   CARD32 samplerstate[6];
+   uint32_t mapstate[6];
+   uint32_t samplerstate[6];
 
    Bool directRenderingDisabled;	/* DRI disabled in PreInit. */
    Bool directRenderingEnabled;		/* DRI enabled this generation. */
@@ -529,8 +550,8 @@ typedef struct _I830Rec {
    Bool StolenOnly;
 
    Bool swfSaved;
-   CARD32 saveSWF0;
-   CARD32 saveSWF4;
+   uint32_t saveSWF0;
+   uint32_t saveSWF4;
 
    Bool checkDevices;
 
@@ -553,70 +574,70 @@ typedef struct _I830Rec {
 
    enum backlight_control backlight_control_method;
 
-   CARD32 saveDSPACNTR;
-   CARD32 saveDSPBCNTR;
-   CARD32 savePIPEACONF;
-   CARD32 savePIPEBCONF;
-   CARD32 savePIPEASRC;
-   CARD32 savePIPEBSRC;
-   CARD32 saveFPA0;
-   CARD32 saveFPA1;
-   CARD32 saveDPLL_A;
-   CARD32 saveDPLL_A_MD;
-   CARD32 saveHTOTAL_A;
-   CARD32 saveHBLANK_A;
-   CARD32 saveHSYNC_A;
-   CARD32 saveVTOTAL_A;
-   CARD32 saveVBLANK_A;
-   CARD32 saveVSYNC_A;
-   CARD32 saveBCLRPAT_A;
-   CARD32 saveDSPASTRIDE;
-   CARD32 saveDSPASIZE;
-   CARD32 saveDSPAPOS;
-   CARD32 saveDSPABASE;
-   CARD32 saveDSPASURF;
-   CARD32 saveDSPATILEOFF;
-   CARD32 saveFPB0;
-   CARD32 saveFPB1;
-   CARD32 saveDPLL_B;
-   CARD32 saveDPLL_B_MD;
-   CARD32 saveHTOTAL_B;
-   CARD32 saveHBLANK_B;
-   CARD32 saveHSYNC_B;
-   CARD32 saveVTOTAL_B;
-   CARD32 saveVBLANK_B;
-   CARD32 saveVSYNC_B;
-   CARD32 saveBCLRPAT_B;
-   CARD32 saveDSPBSTRIDE;
-   CARD32 saveDSPBSIZE;
-   CARD32 saveDSPBPOS;
-   CARD32 saveDSPBBASE;
-   CARD32 saveDSPBSURF;
-   CARD32 saveDSPBTILEOFF;
-   CARD32 saveVCLK_DIVISOR_VGA0;
-   CARD32 saveVCLK_DIVISOR_VGA1;
-   CARD32 saveVCLK_POST_DIV;
-   CARD32 saveVGACNTRL;
-   CARD32 saveADPA;
-   CARD32 saveLVDS;
-   CARD32 saveDVOA;
-   CARD32 saveDVOB;
-   CARD32 saveDVOC;
-   CARD32 savePP_ON;
-   CARD32 savePP_OFF;
-   CARD32 savePP_CONTROL;
-   CARD32 savePP_CYCLE;
-   CARD32 savePFIT_CONTROL;
-   CARD32 savePaletteA[256];
-   CARD32 savePaletteB[256];
-   CARD32 saveSWF[17];
-   CARD32 saveBLC_PWM_CTL;
-   CARD32 saveBLC_PWM_CTL2;
-   CARD32 saveFBC_CFB_BASE;
-   CARD32 saveFBC_LL_BASE;
-   CARD32 saveFBC_CONTROL2;
-   CARD32 saveFBC_CONTROL;
-   CARD32 saveFBC_FENCE_OFF;
+   uint32_t saveDSPACNTR;
+   uint32_t saveDSPBCNTR;
+   uint32_t savePIPEACONF;
+   uint32_t savePIPEBCONF;
+   uint32_t savePIPEASRC;
+   uint32_t savePIPEBSRC;
+   uint32_t saveFPA0;
+   uint32_t saveFPA1;
+   uint32_t saveDPLL_A;
+   uint32_t saveDPLL_A_MD;
+   uint32_t saveHTOTAL_A;
+   uint32_t saveHBLANK_A;
+   uint32_t saveHSYNC_A;
+   uint32_t saveVTOTAL_A;
+   uint32_t saveVBLANK_A;
+   uint32_t saveVSYNC_A;
+   uint32_t saveBCLRPAT_A;
+   uint32_t saveDSPASTRIDE;
+   uint32_t saveDSPASIZE;
+   uint32_t saveDSPAPOS;
+   uint32_t saveDSPABASE;
+   uint32_t saveDSPASURF;
+   uint32_t saveDSPATILEOFF;
+   uint32_t saveFPB0;
+   uint32_t saveFPB1;
+   uint32_t saveDPLL_B;
+   uint32_t saveDPLL_B_MD;
+   uint32_t saveHTOTAL_B;
+   uint32_t saveHBLANK_B;
+   uint32_t saveHSYNC_B;
+   uint32_t saveVTOTAL_B;
+   uint32_t saveVBLANK_B;
+   uint32_t saveVSYNC_B;
+   uint32_t saveBCLRPAT_B;
+   uint32_t saveDSPBSTRIDE;
+   uint32_t saveDSPBSIZE;
+   uint32_t saveDSPBPOS;
+   uint32_t saveDSPBBASE;
+   uint32_t saveDSPBSURF;
+   uint32_t saveDSPBTILEOFF;
+   uint32_t saveVCLK_DIVISOR_VGA0;
+   uint32_t saveVCLK_DIVISOR_VGA1;
+   uint32_t saveVCLK_POST_DIV;
+   uint32_t saveVGACNTRL;
+   uint32_t saveADPA;
+   uint32_t saveLVDS;
+   uint32_t saveDVOA;
+   uint32_t saveDVOB;
+   uint32_t saveDVOC;
+   uint32_t savePP_ON;
+   uint32_t savePP_OFF;
+   uint32_t savePP_CONTROL;
+   uint32_t savePP_CYCLE;
+   uint32_t savePFIT_CONTROL;
+   uint32_t savePaletteA[256];
+   uint32_t savePaletteB[256];
+   uint32_t saveSWF[17];
+   uint32_t saveBLC_PWM_CTL;
+   uint32_t saveBLC_PWM_CTL2;
+   uint32_t saveFBC_CFB_BASE;
+   uint32_t saveFBC_LL_BASE;
+   uint32_t saveFBC_CONTROL2;
+   uint32_t saveFBC_CONTROL;
+   uint32_t saveFBC_FENCE_OFF;
 
    enum last_3d *last_3d;
 
@@ -678,6 +699,8 @@ extern void i830_crtc_dpms_video(xf86CrtcPtr crtc, Bool on);
 int
 i830_crtc_pipe (xf86CrtcPtr crtc);
 
+extern xf86CrtcPtr i830_pipe_to_crtc(ScrnInfoPtr pScrn, int pipe);
+
 Bool
 i830_pipe_a_require_activate (ScrnInfoPtr scrn);
 
@@ -696,7 +719,9 @@ extern Bool I830DRIFinishScreenInit(ScreenPtr pScreen);
 extern void I830DRIUnlock(ScrnInfoPtr pScrn);
 extern Bool I830DRILock(ScrnInfoPtr pScrn);
 extern Bool I830DRISetVBlankInterrupt (ScrnInfoPtr pScrn, Bool on);
-Bool i830_update_dri_buffers(ScrnInfoPtr pScrn);
+extern Bool i830_update_dri_buffers(ScrnInfoPtr pScrn);
+extern Bool I830DRISetHWS(ScrnInfoPtr pScrn);
+extern Bool I830DRIInstIrqHandler(ScrnInfoPtr pScrn);
 #endif
 
 unsigned long intel_get_pixmap_offset(PixmapPtr pPix);
@@ -734,6 +759,10 @@ extern long I830CheckAvailableMemory(ScrnInfoPtr pScrn);
 Bool i830_allocate_2d_memory(ScrnInfoPtr pScrn);
 Bool i830_allocate_texture_memory(ScrnInfoPtr pScrn);
 Bool i830_allocate_3d_memory(ScrnInfoPtr pScrn);
+#ifdef INTEL_XVMC
+Bool i830_allocate_xvmc_buffer(ScrnInfoPtr pScrn, const char *name,
+                               i830_memory **buffer, unsigned long size, int flags);
+#endif
 
 extern Bool I830IsPrimary(ScrnInfoPtr pScrn);
 
@@ -778,6 +807,9 @@ Bool i830_check_composite(int op, PicturePtr pSrc, PicturePtr pMask,
 Bool i830_prepare_composite(int op, PicturePtr pSrc, PicturePtr pMask,
 			    PicturePtr pDst, PixmapPtr pSrcPixmap,
 			    PixmapPtr pMaskPixmap, PixmapPtr pDstPixmap);
+Bool
+i830_transform_is_affine (PictTransformPtr t);
+
 void i830_composite(PixmapPtr pDst, int srcX, int srcY,
 		    int maskX, int maskY, int dstX, int dstY, int w, int h);
 void i830_done_composite(PixmapPtr pDst);
@@ -796,10 +828,14 @@ Bool i965_prepare_composite(int op, PicturePtr pSrc, PicturePtr pMask,
 void i965_composite(PixmapPtr pDst, int srcX, int srcY,
 		    int maskX, int maskY, int dstX, int dstY, int w, int h);
 
-void
+Bool
 i830_get_transformed_coordinates(int x, int y, PictTransformPtr transform,
 				 float *x_out, float *y_out);
 
+Bool
+i830_get_transformed_coordinates_3d(int x, int y, PictTransformPtr transform,
+				    float *x_out, float *y_out, float *z_out);
+
 void i830_enter_render(ScrnInfoPtr);
 
 static inline int i830_fb_compression_supported(I830Ptr pI830)
@@ -818,6 +854,13 @@ static inline int i830_fb_compression_supported(I830Ptr pI830)
 
 Bool i830_pixmap_tiled(PixmapPtr p);
 
+/* Batchbuffer compatibility handling */
+#define BEGIN_BATCH(n) BEGIN_LP_RING(n)
+#define ENSURE_BATCH(n)
+#define OUT_BATCH(d) OUT_RING(d)
+#define OUT_BATCH_F(x) OUT_RING_F(x)
+#define ADVANCE_BATCH() ADVANCE_LP_RING()
+
 extern const int I830PatternROP[16];
 extern const int I830CopyROP[16];
 
@@ -838,6 +881,7 @@ extern const int I830CopyROP[16];
 #define QUIRK_IGNORE_LVDS		0x00000002
 #define QUIRK_IGNORE_MACMINI_LVDS 	0x00000004
 #define QUIRK_PIPEA_FORCE		0x00000008
+#define QUIRK_IVCH_NEED_DVOB		0x00000010
 extern void i830_fixup_devices(ScrnInfoPtr);
 
 #endif /* _I830_H_ */
